Port equ 0ffh
tempo equ 220

dseg
SoundProc_: public SoundProc_
    defw 0

cseg

InitSound_: public InitSound_
    push hl
        ld hl,0
        ld (SoundProc_),hl
    pop hl
ret


CallSound:
jp (hl)

CallSound_: public CallSound_
    push af | push hl
        ld hl,(SoundProc_)
        ld a,l
        or h
        if nz
            call CallSound
            ld hl,0
            ld (SoundProc_),hl
        endif
    pop hl | pop af
ret


Pulse:
; in:
;   bc = cycle
    ld a,5
    out (Port), a
    push bc
        do
            dec bc
            ld a,c
            or b
        while nz | wend
    pop bc
    ld a,4
    out (Port), a
    push bc
        do
            dec bc
            ld a,c
            or b
        while nz | wend
    pop bc
ret 


Tone:
; in:
;   b = cycle
;   hl = length
    push de
        do
            call Pulse
            or a
            sbc hl,bc
        while nc | wend
    pop de
ret 


Dummy:
; in:
;   bc = cycle
    push bc
        do
            dec bc
            ld a,c
            or b
        while nz | wend
    pop bc
    push bc
        do
            dec bc
            ld a,c
            or b
        while nz | wend
    pop bc
ret 

Rest:
; in:
;   b = cycle
;   hl = length
    push de
        do
            call Dummy
            or a
            sbc hl,bc
        while nc | wend
    pop de
ret 

Sound_Loose:
    push af | push bc
        ld bc, 200/6
        do
            Sound_Loose.Loop:
            call Pulse
            
            ld a,c
            add a,2*6
            ld c,a
            ld a,b
            adc a,0
            ld b,a
            
            or a
            jr z,Sound_Loose.Loop
            ld a,c
            cp low 300/6
        while c | wend
    pop bc | pop af
ret 

Sound_Loose_:  public Sound_Loose_
    push hl
        ld hl,Sound_Loose
        ld (SoundProc_),hl
    pop hl
ret


MakeLangth:
    ld h,a
    ld l,0
    ld d,h
    ld e,l
    add hl,hl ;*2
    add hl,de ;*3
    ; add hl,hl ;*6
    ; add hl,de ;*7
    ; add hl,hl ;*14
ret

Melody: 
; in
;   hl: address
    push af
    push de
    push bc
        do
            ld c,(hl) | inc hl
            ld b,(hl) | inc hl
            ld a,c
            or a
        while nz
            if nz
                ld a,b
                or a
                if nz
                    push hl
                        ld a,c
                        dec b
                        ld l,b
                        ld h,0
                        add hl,hl
                        ld de,cycles
                        add hl,de
                        ld c,(hl)
                        inc hl
                        ld b,(hl)
                    pop hl
                    push hl
                        call MakeLangth
                        call Tone
                    pop hl
                else
                    push hl
                        ld a,c
                        ld bc,100
                        call MakeLangth
                        call Rest
                    pop hl
                endif
            endif
        wend
        xor a
        out (Port),a
    pop bc
    pop de
    pop af
ret


hit_notes:
    defb 1,F4, 1,G4, 1,A4, 1,B4, 1,C5, 1,D5, 1,E5, 1,F5
    defb 0 
Sound_Hit: 
    push af | push hl
        ld hl,hit_notes
        call Melody
    pop hl | pop af
ret

Sound_Hit_:  public Sound_Hit_
    push hl
        ld hl,Sound_Hit
        ld (SoundProc_),hl
    pop hl
ret


fire_notes:
    defb 1,F5, 1,D5S, 1,C5S, 1,B4, 1,A4, 1,G5, 0
Sound_Fire_: public Sound_Fire_
    push af | push hl
        ld hl,fire_notes
        call Melody
    pop hl | pop af
ret


; beep_notes:
;     defb 1,A4, 1,0, 0
; Sound_Beep_:  public Sound_Beep_
;     push hl
;         ld hl,beep_notes
;         call Melody
;     pop hl
; ret


up_notes:
    defb 1,C4, 1, C4S, 1,D4, 1,F4, 1,A4, 1,C5, 0
    defb 0 
Sound_Up: 
    push af | push hl
        ld hl,up_notes
        call Melody
    pop hl | pop af
ret

Sound_Up_:  public Sound_Up_
    push hl
        ld hl,Sound_Up
        ld (SoundProc_),hl
    pop hl
ret



start_notes: 
	defb 5,A4,1,0, 11,B4,1,0, 11,C5,1,0
	defb 11,C5,1,0, 5,A4,1,0, 11,D5,1,0
	defb 11,D5,1,0, 5,C5,1,0, 17,D5,1,0
	defb 35,E5,1,0
	defb 12,0
    defb 0
Sound_Start_:
    public Sound_Start_
    push hl
        ld hl,start_notes
        call Melody
        pop hl
ret

; clear_notes: 
; 	defb 5,A4,1,0, 5,A4,1,0, 5,G4,1,0, 5,F4,1,0
; 	defb 5,G4,1,0, 11,A4,1,0, 11,B4,1,0
; 	defb 5,B4,1,0, 5,A4,1,0, 5,G4,1,0, 5,A4,1,0
; 	defb 11,B4,1,0, 29,C5,1,0
; 	defb 24,0
;     defb 0
; Sound_Clear_:
;     public Sound_Clear_
;     push hl
;         ld hl,clear_notes
;         call Melody
;     pop hl
; ret

over_notes: 
	defb 11,A4,1,0, 5,E4,1,0, 5,A4,1,0
	defb 5,G4,1,0, 5,F4,1,0, 5,E4,1,0, 5,D4,1,0
	defb 35,E4,1,0
	defb 12,0
    defb 0
Sound_GameOver_:
    public Sound_GameOver_
    push hl
        ld hl,over_notes
        call Melody
    pop hl
ret


C4 equ 1
C4S equ 2
D4 equ 3
D4S equ 4
E4 equ 5
F4 equ 6
F4S equ 7
G4 equ 8
G4S equ 9
A4 equ 10
A4S equ 11
B4 equ 12
C5 equ 13
C5S equ 14
D5 equ 15
D5S equ 16
E5 equ 17
F5 equ 18
F5S equ 19
G5 equ 20

cycles:
defw 130   ;C4
defw 123   ;C#4
defw 116   ;D4
defw 110   ;D#4
defw 103   ;E4
defw 98   ;F4
defw 92   ;F#4
defw 87   ;G4
defw 82   ;G#4
defw 77   ;A4
defw 73   ;A#4
defw 69   ;B4
defw 65   ;C5
defw 61   ;C#5
defw 58   ;D5
defw 55   ;D#5
defw 51   ;E5
defw 49   ;F5
defw 46   ;F#5
defw 43   ;G5
