#include "Status.h"
#include "Vram.h"
#include "Print.h"
#include "Chars.h"
#include "Sprite.h"

constexpr byte Char_Remain = Char_Fighter;

static ptr<byte> PrintS(ptr<byte> pVram, ptr<byte> p)
{
    byte c;
    while ((c = *p) != 0) {
        pVram = PrintC(pVram, c);
        ++p;
    }
    return pVram;
}

void PrintStatus()
{
    PrintS(Vram + VramRowSize * 0 + 56 * VramStep, "SCORE");
    PrintS(Vram + VramRowSize * 3 + 56 * VramStep, "HIGH");
    PrintS(Vram + VramRowSize * 4 + 56 * VramStep, "SCORE");
    PrintS(Vram + VramRowSize * 7 + 56 * VramStep, "STAGE");
    PrintByteNumber2(Vram + VramRowSize * 8 + 62 * VramStep, CurrentStage + 1);
    PrintRemain();
    PrintScore();
}

void PrintScore()
{
    ptr<byte> pVram;
    
    pVram = PrintNumber5(Vram + VramRowSize * 1 + 58 * VramStep, Score);
    PrintC(pVram, '0');

    pVram = PrintNumber5(Vram + VramRowSize * 5 + 58 * VramStep, HiScore);
    PrintC(pVram, '0');
}

void PrintRemain()
{
    static const byte[] RemainChars = { 0xe0, 0xfe, 0xf4, 0x80, 0xc3, 0xc2, 0xc2, 0xc1, };
    ptr<byte> pVram;
    pVram = Vram + VramRowSize * 14 + 58 * VramStep;
    if (RemainCount > 1) {
        pVram = Put2S(pVram, RemainChars);
        if (RemainCount > 2) {
            pVram = PrintC(pVram, RemainCount + ('0' - 1));
        }
        else {
            pVram = PrintC(pVram, ' ');
        }
    }
    else {
        pVram = Erase2(pVram);
    }
}


void PrintGameOver()
{
    constexpr byte Left = (56 - 8) / 2;
    PrintS(Vram + VramRowSize * (VramHeight / 2) + Left * VramStep, "GAME OVER");
}

void Title()
{
    ClearScreen(); 
    HideAllSprites();
    // UpdateSprites();
    PrintStatus();
    {
        static const byte[] TitleBytes = {
            //	G
            0xe0, 0xde, 0xc3, 0xcd, 
            0xca, 0xf5, 0xc3, 0xff, 
            0xc0, 0xc2, 0xc3, 0xc3, 
            //	U
            0xea, 0xd5, 0xc0, 0xff, 
            0xea, 0xd5, 0xc0, 0xff, 
            0xc0, 0xc3, 0xc3, 0xc1, 
            //	N
            0xea, 0xfd, 0xd0, 0xff, 
            0xea, 0xd7, 0xef, 0xff, 
            0xc2, 0xc1, 0xc0, 0xc3, 
            //	T
            0xc2, 0xeb, 0xd7, 0xc1, 
            0xc0, 0xea, 0xd5, 0xc0, 
            0xc0, 0xc2, 0xc1, 0xc0, 
            //	U
            0xea, 0xd5, 0xc0, 0xff, 
            0xea, 0xd5, 0xc0, 0xff, 
            0xc0, 0xc3, 0xc3, 0xc1, 
            //	S
            0xe8, 0xd7, 0xc3, 0xcd, 
            0xe0, 0xd3, 0xc3, 0xfd, 
            0xc0, 0xc3, 0xc3, 0xc1, 
        };
        constexpr byte LogoLength = 6;
        constexpr byte LogoWidth = 4 * LogoLength;
        constexpr byte LogoX = (56 - LogoWidth) / 2;

        ptr<byte> p;
        ptr<byte> pVram;
        pVram = Vram + VramRowSize * 5 + LogoX;
        p = TitleBytes;
        repeat (LogoLength) {
            repeat (3) {
                repeat (4) {
                    *pVram = *p | 0xc0;
                    ++pVram;
                    ++p;
                }
                pVram += VramRowSize - 4;
            }
            pVram += 4 - VramRowSize * 3;
        }
    }
    PrintS(Vram + VramRowSize * 13 + (56 - 14) * VramStep / 2, "PUSH SPACE KEY");
    PrintS(Vram + VramRowSize * 14 + (56 - 14) * VramStep / 2, "OR ENTER KEY");
    PrintS(Vram + VramRowSize * 15 + (56 - 12) * VramStep, "INUFUTO 2022");
    // {
    //     byte c;
    //     ptr<byte> pVram;
    //     vram = Vram;
    //     c = 0;
    //     repeat (0) {
    //         vram = Put(pVram, c);
    //         ++c;
    //         if ((c & 15) == 0) {
    //             vram += VramRowSize - 16;
    //         }
    //     }
    // }
}


void AddScore(word pts)
{
    Score += pts;
    if (Score > HiScore) {
        HiScore = Score;
    }
    PrintScore();
}
